/**
 * @file   parseDomset.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Sun Aug  8 18:40:05 2004
 * 
 * @brief  Parse a dominating set input and place in a table.
 * 
 * 
 */

#include <stdio.h>
#include "inescan.h"
#include "int_table.h"

/** 
 * match a single token
 * 
 * @param token token to match
 * @param msg error message if not matched
 */

static void expect(int token,const char *msg){
  int next_token;
  if ((next_token=inelex())!=token){
    fprintf(stderr,"line %d: expected token %d got %d  %s",
	    ineline,token, next_token,msg);
    exit(1);
  }
}


/** 
 * parse the given input stream
 * 
 * @param input open file stream
 * 
 * @return a table containing the set data
 */
int_table_p parseDomset(FILE *input){
  int_table_p  table;
  int token;
  int row_count;
  int dimension;
  int i;

  /* scan for begin */
  for (token=inelex(); token>=0 && token != INE_BEGIN; token=inelex());
  if (token<0) 
    fatal("No begin found");

  expect(INE_EOL, "extra junk on line");

  expect(INE_INT,"no row count");
  row_count=ineint;

  expect(INE_INT,"no dimension");
  /* take out the reference point */
  dimension=ineint-1;

  table=it_new(row_count);

  for (i=0; i< row_count; i++){
    int row_size,j;

    expect(INE_INT,"missing row size");
    row_size=ineint;

    expect(':',"expected :");
    
    for (j=0; j<row_size; j++){

      expect(INE_INT,"index expected");
      /* this is not so nice, since it hard codes the fact the point 1 
	 is the reference point */
      it_set(table,i,j,ineint-2);
    }
    expect(INE_EOL, "extra junk on line");
  }

  return new;
  
}  
